package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.dict.domain.IBZDictItem;
import cn.ibizlab.core.dict.service.IIBZDictItemService;
import cn.ibizlab.core.dict.filter.IBZDictItemSearchContext;

@Slf4j
@Api(tags = {"字典项目" })
@RestController("api-ibzdictitem")
@RequestMapping("")
public class IBZDictItemResource {

    @Autowired
    public IIBZDictItemService ibzdictitemService;

    @Autowired
    @Lazy
    public IBZDictItemMapping ibzdictitemMapping;

    @PostAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(returnObject.body),'ibzdict-IBZDictItem-Get')")
    @ApiOperation(value = "获取字典项目", tags = {"字典项目" },  notes = "获取字典项目")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> get(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "获取字典项目草稿", tags = {"字典项目" },  notes = "获取字典项目草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdictitems/getdraft")
    public ResponseEntity<IBZDictItemDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemMapping.toDto(ibzdictitemService.getDraft(new IBZDictItem())));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdto),'ibzdict-IBZDictItem-Save')")
    @ApiOperation(value = "保存字典项目", tags = {"字典项目" },  notes = "保存字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/save")
    public ResponseEntity<Boolean> save(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.save(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos),'ibzdict-IBZDictItem-Save')")
    @ApiOperation(value = "批量保存字典项目", tags = {"字典项目" },  notes = "批量保存字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.saveBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查字典项目", tags = {"字典项目" },  notes = "检查字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.checkKey(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.get(#ibzdictitem_id),'ibzdict-IBZDictItem-Remove')")
    @ApiOperation(value = "删除字典项目", tags = {"字典项目" },  notes = "删除字典项目")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
         return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.remove(ibzdictitem_id));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.getIbzdictitemByIds(#ids),'ibzdict-IBZDictItem-Remove')")
    @ApiOperation(value = "批量删除字典项目", tags = {"字典项目" },  notes = "批量删除字典项目")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        ibzdictitemService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdto),'ibzdict-IBZDictItem-Create')")
    @ApiOperation(value = "新建字典项目", tags = {"字典项目" },  notes = "新建字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> create(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos),'ibzdict-IBZDictItem-Create')")
    @ApiOperation(value = "批量新建字典项目", tags = {"字典项目" },  notes = "批量新建字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.createBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.get(#ibzdictitem_id),'ibzdict-IBZDictItem-Update')")
    @ApiOperation(value = "更新字典项目", tags = {"字典项目" },  notes = "更新字典项目")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> update(@PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
		IBZDictItem domain  = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain .setItemid(ibzdictitem_id);
		ibzdictitemService.update(domain );
		IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.getIbzdictitemByEntities(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos)),'ibzdict-IBZDictItem-Update')")
    @ApiOperation(value = "批量更新字典项目", tags = {"字典项目" },  notes = "批量更新字典项目")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdictitems/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        ibzdictitemService.updateBatch(ibzdictitemMapping.toDomain(ibzdictitemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDictItem-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"字典项目" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdictitems/fetchdefault")
	public ResponseEntity<List<IBZDictItemDTO>> fetchDefault(IBZDictItemSearchContext context) {
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        List<IBZDictItemDTO> list = ibzdictitemMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDictItem-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"字典项目" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/ibzdictitems/searchdefault")
	public ResponseEntity<Page<IBZDictItemDTO>> searchDefault(@RequestBody IBZDictItemSearchContext context) {
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictitemMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
    @PostAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(returnObject.body),'ibzdict-IBZDictItem-Get')")
    @ApiOperation(value = "根据数据字典获取字典项目", tags = {"字典项目" },  notes = "根据数据字典获取字典项目")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> getByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "根据数据字典获取字典项目草稿", tags = {"字典项目" },  notes = "根据数据字典获取字典项目草稿")
    @RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/getdraft")
    public ResponseEntity<IBZDictItemDTO> getDraftByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id) {
        IBZDictItem domain = new IBZDictItem();
        domain.setDictid(ibzdict_id);
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemMapping.toDto(ibzdictitemService.getDraft(domain)));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdto),'ibzdict-IBZDictItem-Save')")
    @ApiOperation(value = "根据数据字典保存字典项目", tags = {"字典项目" },  notes = "根据数据字典保存字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/save")
    public ResponseEntity<Boolean> saveByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.save(domain));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos),'ibzdict-IBZDictItem-Save')")
    @ApiOperation(value = "根据数据字典批量保存字典项目", tags = {"字典项目" },  notes = "根据数据字典批量保存字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/savebatch")
    public ResponseEntity<Boolean> saveBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
             domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.saveBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "根据数据字典检查字典项目", tags = {"字典项目" },  notes = "根据数据字典检查字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKeyByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.checkKey(ibzdictitemMapping.toDomain(ibzdictitemdto)));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.get(#ibzdictitem_id),'ibzdict-IBZDictItem-Remove')")
    @ApiOperation(value = "根据数据字典删除字典项目", tags = {"字典项目" },  notes = "根据数据字典删除字典项目")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> removeByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
		return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemService.remove(ibzdictitem_id));
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.getIbzdictitemByIds(#ids),'ibzdict-IBZDictItem-Remove')")
    @ApiOperation(value = "根据数据字典批量删除字典项目", tags = {"字典项目" },  notes = "根据数据字典批量删除字典项目")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> removeBatchByIBZDict(@RequestBody List<String> ids) {
        ibzdictitemService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdto),'ibzdict-IBZDictItem-Create')")
    @ApiOperation(value = "根据数据字典建立字典项目", tags = {"字典项目" },  notes = "根据数据字典建立字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> createByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos),'ibzdict-IBZDictItem-Create')")
    @ApiOperation(value = "根据数据字典批量建立字典项目", tags = {"字典项目" },  notes = "根据数据字典批量建立字典项目")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> createBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
            domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.createBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.get(#ibzdictitem_id),'ibzdict-IBZDictItem-Update')")
    @ApiOperation(value = "根据数据字典更新字典项目", tags = {"字典项目" },  notes = "根据数据字典更新字典项目")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> updateByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = ibzdictitemMapping.toDomain(ibzdictitemdto);
        domain.setDictid(ibzdict_id);
        domain.setItemid(ibzdictitem_id);
		ibzdictitemService.update(domain);
        IBZDictItemDTO dto = ibzdictitemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictitemService.getIbzdictitemByEntities(this.ibzdictitemMapping.toDomain(#ibzdictitemdtos)),'ibzdict-IBZDictItem-Update')")
    @ApiOperation(value = "根据数据字典批量更新字典项目", tags = {"字典项目" },  notes = "根据数据字典批量更新字典项目")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}/ibzdictitems/batch")
    public ResponseEntity<Boolean> updateBatchByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody List<IBZDictItemDTO> ibzdictitemdtos) {
        List<IBZDictItem> domainlist=ibzdictitemMapping.toDomain(ibzdictitemdtos);
        for(IBZDictItem domain:domainlist){
            domain.setDictid(ibzdict_id);
        }
        ibzdictitemService.updateBatch(domainlist);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDictItem-Default-all')")
	@ApiOperation(value = "根据数据字典获取DEFAULT", tags = {"字典项目" } ,notes = "根据数据字典获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/{ibzdict_id}/ibzdictitems/fetchdefault")
	public ResponseEntity<List<IBZDictItemDTO>> fetchIBZDictItemDefaultByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id,IBZDictItemSearchContext context) {
        context.setN_dictid_eq(ibzdict_id);
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        List<IBZDictItemDTO> list = ibzdictitemMapping.toDto(domains.getContent());
	    return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDictItem-Default-all')")
	@ApiOperation(value = "根据数据字典查询DEFAULT", tags = {"字典项目" } ,notes = "根据数据字典查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/ibzdicts/{ibzdict_id}/ibzdictitems/searchdefault")
	public ResponseEntity<Page<IBZDictItemDTO>> searchIBZDictItemDefaultByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemSearchContext context) {
        context.setN_dictid_eq(ibzdict_id);
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictitemMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

