package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.dict.domain.IBZDict;
import cn.ibizlab.core.dict.service.IIBZDictService;
import cn.ibizlab.core.dict.filter.IBZDictSearchContext;

@Slf4j
@Api(tags = {"数据字典" })
@RestController("api-ibzdict")
@RequestMapping("")
public class IBZDictResource {

    @Autowired
    public IIBZDictService ibzdictService;

    @Autowired
    @Lazy
    public IBZDictMapping ibzdictMapping;

    @PostAuthorize("hasPermission(this.ibzdictMapping.toDomain(returnObject.body),'ibzdict-IBZDict-Get')")
    @ApiOperation(value = "获取数据字典", tags = {"数据字典" },  notes = "获取数据字典")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/{ibzdict_id}")
    public ResponseEntity<IBZDictDTO> get(@PathVariable("ibzdict_id") String ibzdict_id) {
        IBZDict domain = ibzdictService.get(ibzdict_id);
        IBZDictDTO dto = ibzdictMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictMapping.toDomain(#ibzdictdto),'ibzdict-IBZDict-Create')")
    @ApiOperation(value = "新建数据字典", tags = {"数据字典" },  notes = "新建数据字典")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts")
    @Transactional
    public ResponseEntity<IBZDictDTO> create(@RequestBody IBZDictDTO ibzdictdto) {
        IBZDict domain = ibzdictMapping.toDomain(ibzdictdto);
		ibzdictService.create(domain);
        IBZDictDTO dto = ibzdictMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictMapping.toDomain(#ibzdictdtos),'ibzdict-IBZDict-Create')")
    @ApiOperation(value = "批量新建数据字典", tags = {"数据字典" },  notes = "批量新建数据字典")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.createBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查数据字典", tags = {"数据字典" },  notes = "检查数据字典")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody IBZDictDTO ibzdictdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(ibzdictService.checkKey(ibzdictMapping.toDomain(ibzdictdto)));
    }

    @ApiOperation(value = "获取数据字典草稿", tags = {"数据字典" },  notes = "获取数据字典草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/ibzdicts/getdraft")
    public ResponseEntity<IBZDictDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictMapping.toDto(ibzdictService.getDraft(new IBZDict())));
    }

    @PreAuthorize("hasPermission(this.ibzdictService.get(#ibzdict_id),'ibzdict-IBZDict-Update')")
    @ApiOperation(value = "更新数据字典", tags = {"数据字典" },  notes = "更新数据字典")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/{ibzdict_id}")
    @Transactional
    public ResponseEntity<IBZDictDTO> update(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictDTO ibzdictdto) {
		IBZDict domain  = ibzdictMapping.toDomain(ibzdictdto);
        domain .setDictid(ibzdict_id);
		ibzdictService.update(domain );
		IBZDictDTO dto = ibzdictMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission(this.ibzdictService.getIbzdictByEntities(this.ibzdictMapping.toDomain(#ibzdictdtos)),'ibzdict-IBZDict-Update')")
    @ApiOperation(value = "批量更新数据字典", tags = {"数据字典" },  notes = "批量更新数据字典")
	@RequestMapping(method = RequestMethod.PUT, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.updateBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictService.get(#ibzdict_id),'ibzdict-IBZDict-Remove')")
    @ApiOperation(value = "删除数据字典", tags = {"数据字典" },  notes = "删除数据字典")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/{ibzdict_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("ibzdict_id") String ibzdict_id) {
         return ResponseEntity.status(HttpStatus.OK).body(ibzdictService.remove(ibzdict_id));
    }

    @PreAuthorize("hasPermission(this.ibzdictService.getIbzdictByIds(#ids),'ibzdict-IBZDict-Remove')")
    @ApiOperation(value = "批量删除数据字典", tags = {"数据字典" },  notes = "批量删除数据字典")
	@RequestMapping(method = RequestMethod.DELETE, value = "/ibzdicts/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        ibzdictService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasPermission(this.ibzdictMapping.toDomain(#ibzdictdto),'ibzdict-IBZDict-Save')")
    @ApiOperation(value = "保存数据字典", tags = {"数据字典" },  notes = "保存数据字典")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/save")
    public ResponseEntity<Boolean> save(@RequestBody IBZDictDTO ibzdictdto) {
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictService.save(ibzdictMapping.toDomain(ibzdictdto)));
    }

    @PreAuthorize("hasPermission(this.ibzdictMapping.toDomain(#ibzdictdtos),'ibzdict-IBZDict-Save')")
    @ApiOperation(value = "批量保存数据字典", tags = {"数据字典" },  notes = "批量保存数据字典")
	@RequestMapping(method = RequestMethod.POST, value = "/ibzdicts/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<IBZDictDTO> ibzdictdtos) {
        ibzdictService.saveBatch(ibzdictMapping.toDomain(ibzdictdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDict-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"数据字典" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/ibzdicts/fetchdefault")
	public ResponseEntity<List<IBZDictDTO>> fetchDefault(IBZDictSearchContext context) {
        Page<IBZDict> domains = ibzdictService.searchDefault(context) ;
        List<IBZDictDTO> list = ibzdictMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzdict-IBZDict-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"数据字典" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/ibzdicts/searchdefault")
	public ResponseEntity<Page<IBZDictDTO>> searchDefault(@RequestBody IBZDictSearchContext context) {
        Page<IBZDict> domains = ibzdictService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(ibzdictMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

